<?php
/*
 * @package     plg_radicalmart_msgif
 * @version     __DEPLOY_VERSION__
 * @author      Vladislav Tsygankov
 * @copyright   Copyright (c) 2024 MSGroup. All rights reserved.
 * @license     GNU/GPL license: https://www.gnu.org/copyleft/gpl.html
 * @link        https://msgru.com/
 */

namespace MSGJoomla\Plugin\RadicalMart\FilterIndex\Console;

\defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\Component\RadicalMart\Administrator\Console\AbstractCommand;
use Joomla\Component\RadicalMart\Administrator\Helper\CommandsHelper;
use Joomla\Component\RadicalMart\Administrator\Helper\ParamsHelper;
use Joomla\Component\RadicalMart\Administrator\Model\CategoryModel;
use Joomla\Component\RadicalMart\Administrator\Helper\FieldsHelper;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use MSGJoomla\Module\RMFilter\Site\Helper\FilterHelper;
use Joomla\Component\RadicalMart\Administrator\Traits\CategoriesCommandTrait;
use MSGJoomla\Module\RMFilter\Site\Field\SortablefieldsField;

class MSGFilterIndex extends AbstractCommand
{
    use CategoriesCommandTrait;

	/**
	 * The default command name
	 *
	 * @var    string|null
	 *
	 * @since  2.0.0
	 */
	protected static $defaultName = 'radicalmart_filter:index';

	/**
	 * Command text title for configure.
	 *
	 * @var   string
	 *
	 * @since 2.0.0
	 */
	protected string $commandText = 'Filter: Index options';

	/**
	 * Command description for configure help block.
	 *
	 * @var   string
	 *
	 * @since 2.0.0
	 */
	protected string $commandDescription = 'run rebuild categories products and meta products mapping';

	/**
	 * Command methods for step by step run.
	 *
	 * @var  array
	 *
	 * @since 2.0.0
	 */
	protected array $methods = [
		'startFilterIndexing',
	];

    protected function startFilterIndexing()
    {
        $this->ioStyle->text('Filter indexing');
        $this->ioStyle->progressStart($this->categoriesGetTotal());

        $cacheModel = Factory::getApplication()->bootComponent('com_cache')->getMVCFactory()
            ->createModel('Cache', 'Administrator', ['ignore_request' => true]);

        if (!$cacheModel->clean('mod_msg_rmf')) {
            throw new \Exception('Can\'t clear cache');
        }

        $limit = 10;
        $last  = 0;

        $params = new Registry();
        $params->set('fields', self::getFields());

        while (true)
        {
            // Get pks
            $pks   = CommandsHelper::getNextPrimaryKeys('#__radicalmart_categories', $last, $limit);
            $count = count($pks);
            if ($count === 0)
            {
                break;
            }

            // Run method
            foreach ($pks as $pk)
            {
                $last = (int) $pk;
                $params->set('category_id', $pk);

                FilterHelper::getForm($params);

                $this->ioStyle->progressAdvance();
            }

            $this->getDatabase()->disconnect();

            if ($count !== $limit)
            {
                break;
            }
        }

        $this->ioStyle->progressFinish();
    }

    public static function getFields(): array
    {
        $items = FilterHelper::getAllFields();

        $params = ParamsHelper::getComponentParams();

        // Prepare options
        $fields = array();

        foreach (SortablefieldsField::getBasicFields() as $basicField) {
            // Add self made filter fields
            $field = new \stdClass();
            $field->name = $basicField['name'];
            $field->title = Text::_($basicField['title']);
            $field->custom_title = Text::_($basicField['title']);
            $field->group = 'Filter';
            $field->fieldset = '';
            $field->fieldset_name = '';
            $field->type = $basicField['type'];

            switch ($basicField['type'])
            {
                case "InStock":
                    $field->state = (bool)$params->get('stock_accounting', 0);
                    break;
                case "Price":
                    $field->state = !$params->get('hide_prices', 0);
                    break;
                default:
                    $field->state = null;
                    break;
            }

            $fields[] = $field;
        }

        foreach ($items as $item) {
            if ($item->area !== 'products'
                || (int)$item->params->get('display_filter', 0) !== 1) continue;

            $field = new \stdClass();
            $field->name = $item->alias;
            $field->title = $item->title;
            $field->custom_title = $item->title;
            $field->group = 'ProductField';
            $field->fieldset = $item->fieldset_alias;
            $field->fieldset_name = $item->fieldset_name;

            $field->type = SortablefieldsField::getCorrectType($item);
            $field->state = $item->state && $item->params->get('display_filter', 1);

            $fields[] = $field;
        }

        return $fields;
    }
}
